	TITLE	'DISKDUMP v2.1 - Dump a disk to a serial device'
	STL	'Headers'
*	DISKDUMP - Dump an H17 disk to a serial device in
*		   H8 Emulator portable disk format.
*
*	by Dave Shaw, November, 2001
*
*	v1.1 -- December, 2001
*		Adds support for disks built with the HUG
*		Enhanced SY: driver (e.g., 80-track and/or 2-sided)
*
*	v1.2 -- July, 2002
*		Supports single-drive systems that are not booted in
*		stand-alone mode
*
*	v2.0 -- November, 2002
*		Skips "skipped" sectors and sectors that can't be read
*		from the disk, substituting appropriate default data:
*			skipped sectors - 000Q
*			binary files	- 377Q 000Q
*			text files	- '?'
*		This version also gets rid of the leading "clip line"
*		so that correctly captured images don't require editing
*		before use.
*
*	v2.1 -- December, 2002
*		Calculates a check byte in the same fashion as the H-17
*		driver, and formats a "check" comment following each
*		sector in the dumped image. The emulator uses this check
*		byte to validate the image at load time. If characters
*		are lost or garbled during the dump, there's a very good
*		chance the emulator will catch it.
*
*	This code is not copyrighted; use, modify, and distribute at will.
*
*	This can be run on any H8/H89 to dump diskettes in emulator portable
*	format for use with either the Macintosh H8 Emulator or Dave
*	Wallace's Virtual H8 for Windows/NT.
*
*	Usage:	At the command line, type the command:
*
*		diskdump <rd>=<dd>
*
*	where:	rd = receiving device (e.g., at:, tt:)
*		dd = directory device (e.g., sy1:, sy2:)
*
*	sample: diskdump at:=sy2:
*
*	You must have another computer attached to the serial device to
*	capture the output to a file. The output is simple ascii text.
*
*	For more detail, see the porable disk format description at
*	http://members.aol.com/heath8080a/design_h17.html/#internal%20h17
*
	XTEXT	HDOS
	XTEXT	HDOSROM
	XTEXT	ASCII

AIO.UNI EQU	41061A		Disk unit number (0, 1, 2)
D.SYDD	EQU	40130A		Addr of jump to driver

DC.REA	EQU	000Q		Read command
DC.RER	EQU	002Q		Read regardless
DC.ABT	EQU	007Q		Abort the driver
DC.MOU	EQU	010Q		Mount command
DC.RDY	EQU	012Q		Check for disk ready

PROT.NO EQU	0		No disk protection
PROT.WP EQU	1		Write protection
PROT.LK EQU	2		Disk locked

PROT	EQU	PROT.WP 	Using plain write protection

*	Label sector offsets

LAB.DIS EQU	3		Label offset of 1st directory block
LAB.GRT EQU	5		Label offset of GRT sector
LAB.SPG EQU	7		Label offset of sectors/group count
LAB.VLT EQU	8		Label offset of volume type

VLT.BOO EQU	1		Bootable volume

LAB.VER EQU	9		Label offset of INIT version
LAB.RGT EQU	10		Label offset of RGT sector
LAB.VFL EQU	16		Label offset of volume flags

VFL.2SD EQU	00000001B	2-sided flag
VFL.80T EQU	00000010B	80-track flag

LAB.LAB EQU	17		Label offset of text label

*	Directory block and directory entry offsets

	ORG	0
DIR.NAM DS	8		File name
DIR.EXT DS	3		File extension
	DS	5
DIR.FGN DS	1		File first group
	DS	1
DIR.LSI DS	1		File last sector index
	DS	4
DIR.LEN EQU	*-DIR.NAM
	ERRNZ	DIR.LEN-23	Better be 23 bytes long

DIR.LNK EQU	510		Link to next block at end of this one

DE.DEL	EQU	377Q		First char = 377Q if file deleted
DE.LAST EQU	376Q		First char = 376Q if rest of dir. empty

*	We're keeping a directory extract in memory. This is used to
*	help determine the file's content type (e.g., binary, text),
*	but is also dumped after the file header.

	ORG	0
DE.NAM	DS	8		File name
DE.EXT	DS	3		File extension
DE.FGN	DS	1		First group number
DE.SIZ	DS	2		File size
DE.LEN	EQU	*-DE.NAM	Length of a directory table entry
	
*	We're keeping a table of sector information, indicating the type of
*	the sector (binary, text, skipped) and the address of the directory
*	table entry for the file that sector is part of. (zero = free chain).

	ORG	0
SEC.TYP DS	1		Sector content type

SEC.BIN EQU	'B'		Binary	  - fill with 377Q 000Q
SEC.TXT EQU	'T'		Text	  - fill with '?????'
SEC.SKP EQU	'X'		Skipped   - fill with 000Q
SEC.FRE EQU	'F'		Free list - fill with 000Q

SEC.DIR DS	2		Address of assoc. dir. entry
SEC.RS	DS	2		Relative sector from file front
SEC.LEN EQU	*-SEC.TYP	Length of a sector table entry

SEC.CNT EQU	1600		Number of entries to keep (80x2 disk)
	STL	'Startup'
	EJECT
	ORG	USERFWA

*	Get command off stack and decode it

BEGIN	EQU	*
	CALL	$TYPTX
	DB	NL,'Disk Dumper v2.1, 1-Dec-2002',NL,ENL

	SCALL	.VERS
	JC	BADVERS
	CPI	20H		HDOS 2.0 required
	JNZ	BADVERS

	LXI	H,0
	DAD	SP		(HL) = top of stack
	MVI	A,200Q
	SUB	L
	MOV	C,A
	MVI	A,42Q
	SBB	H
	MOV	B,A		(BC) = Length of parameter
	ORA	C
	JNZ	PARSE		We have something; parse it

BADFMT	EQU	*
	CALL	$TYPTX
	DB	'Usage:',HT
	DB	'diskdump <to>=<from>',NL,NL
	DB	'Where:',HT
	DB	'to = a serial device (TT: or AT: for example)',NL,HT
	DB	'from = a disk device (SY1: for example)',NL,NL
	DB	'Example:',NL,HT
	DB	'diskdump tt:=sy2:',NL,NL
	DB	'Type ctl-c to cancel, ctl-d to quit.',NL,ENL
	JMP	EXIT

*	Parse the command on the stack. This is expected to be of the form:
*	" dvn:=syn:". This should be 10 or 11 bytes long, counting the
*	trailing null.
*
*	Entry:	(HL) = Start of command on stack (leading space)
*		(BC) = Length of command
*
*	Exit:	To our processing loop, with the output device spec and
*		input unit filed and output unit open.
*
PARSE	EQU	*
	MOV	A,B
	ORA	A
	JNZ	BADFMT		Impossibly long
	MOV	A,C
	CPI	12
	JNC	BADFMT		Too long
	CPI	10
	JC	BADFMT		Too short

	DCR	C		(C)  = Correct length
	INX	H		(HL) = Addr of output device
	LXI	D,OUTPUT	(DE) = Addr of output spec
COPYSPC EQU	*
	MOV	A,M
	STAX	D
	INX	H
	INX	D
	DCR	C
	JZ	BADFMT		VERY ugly!
	CPI	':'
	JNZ	COPYSPC 	Not done yet
	XRA	A
	STAX	D		Terminate device spec
	MOV	A,M
	CPI	'='
	JNZ	BADFMT		Malformed request
	
	INX	H
	DCR	C
	MOV	A,C
	CPI	5		s/b "SYn:",0 on the stack
	JNZ	BADFMT
	
	MOV	A,M
	CPI	'S'
	JNZ	BADFMT
	INX	H
	MOV	A,M
	CPI	'Y'
	JNZ	BADFMT	
	INX	H
	MOV	A,M
	CPI	'0'
	JC	BADFMT		Valid units: 0, 1, 2
	CPI	'3'
	JNC	BADFMT
	STA	INPUT		Save input unit number
	INX	H
	MOV	A,M
	CPI	':'
	JNZ	BADFMT		Incorrect form

*	We have a parsed command line - ready to get started

	MVI	A,-1
	SCALL	.CLEAR		Clear so we can dismount if need be

*	Make sure overlays are in so we can run on a single-drive system.
*	Load drivers if available, ignore errors.
*
*	Obviously, modify the driver loads as necessary for your system.

	MVI	A,0
	SCALL	.LOADO		Load overlay 0, if not already there
	MVI	A,1
	SCALL	.LOADO		Load overlay 1, if not already there
	LXI	H,AT
	SCALL	.LOADD		Load AT: if not already there

	MVI	A,CTLC
	LXI	H,CTLCXIT
	SCALL	.CTLC		Set cancel exit

	LXI	SP,STACK	Reset stack
	MVI	A,I.CSLMD
	MVI	B,CSL.CHR+CSL.ECH
	MOV	C,B
	SCALL	.CONSL		Go to character mode, no echo
	
	LXI	H,-1
	SCALL	.SETTOP 	Find RAM top
	SHLD	MAX.MEM
	SCALL	.SETTOP
	JNC	SETUP

*	Yow - this should NEVER happen!

	PUSH	PSW
	CALL	$TYPTX
	DB	BEL,NL,'Error in .SETTOP -',' '+200Q
	POP	PSW
	MVI	H,NL
	SCALL	.ERROR
	JMP	EXIT

*	Driver file specs

AT	DB	'AT:',0
	STL	'Ready input device and set up for dump'
	EJECT
*	SETUP	- Ready input device and get set up for a dump.
*
*	Entry:	None
*
*	Exit:	None
*
SETUP	EQU	*
	LDA	INPUT
	STA	SETUP.D 	Customize dismount message
	SUI	'0'		Convert to binary
	STA	AIO.UNI 	Set unit number
	MVI	A,DC.ABT
	CALL	D.SYDD		Reset the driver
	
	MVI	A,DC.RDY
	CALL	D.SYDD		Disk ready?
	JC	OPENOUT 	No - a good thing
	
	LXI	H,SY.SPEC
	SCALL	.DMOUN		Dismount device
	CALL	$TYPTX
	DB	NL,'Please remove the disk from SY'
SETUP.D DB	'0'
	DB	':',ENL
	
RDY.1	EQU	*
	MVI	A,DC.RDY
	CALL	D.SYDD
	JNC	RDY.1		Loop until disk removed

*	Open the output driver
*
OPENOUT EQU	*
	LXI	H,OUTBUF
	SHLD	OUTPTR		Init. pointer
	LXI	H,OUTPUT
	LXI	D,OUTDFLT
	XRA	A		Channel=0
	STA	OUTNDX		Clear the output index while I'm at it
	SCALL	.OPENW		Open for write
	JNC	ASK		Get it going

*	Error of some kind - tell user and quit

	PUSH	PSW
	CALL	$TYPTX
	DB	BEL,'Error opening output device ','-'+200Q
	
	MVI	H,NL
	POP	PSW
	SCALL	.ERROR
	JMP	EXIT
	STL	'Main dump loop'
	EJECT
*	Main dump loop
*
*	The major steps here are:
*
*	1) ask for a disk to dump and get it ready
*	2) load label, GRT and RGT
*	3) build directory extract
*	4) build sector dump control table
*	5) print a bunch of header information
*	6) dump the disk
*
*	Errors reading sectors in steps 4 & 5 are handled. But none can
*	be tolerated in the first three steps.
*
	SPACE	2,10
*	Step #1 - Ask for a disk and mount it

ASK	EQU	*
	LXI	H,0
	SHLD	ERR.CNT 	Clear error count
	LDA	INPUT
	STA	MSGUNIT
	SUI	'0'
	STA	AIO.UNI
	CALL	$TYPTX
	DB	NL,'Please insert a new disk in SY'
MSGUNIT DS	1
	DB	':',ENL

	SCALL	.CLRCO
WAIT	EQU	*
	SCALL	.SCIN
	JC	WAIT.1
	CPI	4		Ctl-D?
	JZ	CLOSE		Yes - close and exit
WAIT.1	EQU	*
	MVI	A,DC.RDY
	CALL	D.SYDD
	JC	WAIT		Not ready - wait for it

*	The drive is ready - low-level mount the media

	MVI	A,DC.ABT
	CALL	D.SYDD		Abort driver
	CALL	PAUSE		Pause 1/2 second to let things settle
	LDA	INPUT
	SUI	'0'
	STA	AIO.UNI 	Set unit
	MVI	L,0		(L) = Volume number for mount
	MVI	A,DC.MOU
	CALL	D.SYDD		Mount it
	JC	ER.MOU		Problem with mount - tell user and get out
	SPACE	2,10
*	Step #2 - Load the directory, GRT and RGT.

	LXI	H,9		(HL) = label sector
	LXI	D,LABEL 	(DE) = buffer location
	LXI	B,256		(BC) = length
	MVI	A,DC.REA
	CALL	D.SYDD		Read the label
	JC	ER.REA		Error reading - not good
	
	LDA	LABEL
	MOV	L,A		(L)  = volume number
	MVI	A,DC.MOU
	CALL	D.SYDD		Remount with correct volume
	JC	ER.MOU
	
	LHLD	LABEL+LAB.GRT	(HL) = GRT sector
	LXI	D,GRT
	LXI	B,256
	MVI	A,DC.REA
	CALL	D.SYDD		Read the GRT
	JC	ER.REA		Error reading
	
*	HDOS 2.0 puts the RGT sector number into the label. HDOS 1.6 sets this
*	label field to zero. If the RGT sector index is zero, then it can be
*	found in sector 10.

	LHLD	LABEL+LAB.RGT	(HL) = RGT sector
	MOV	A,H
	ORA	L		Do we have RGT sector?
	JNZ	GOT.RGT 	Yes
	LXI	H,10		Else substitute sector 10
GOT.RGT EQU	*
	LXI	D,RGT
	LXI	B,256
	MVI	A,DC.REA
	CALL	D.SYDD
	JC	ER.REA		Error reading
	
	LXI	H,MSG.VOL
	LDA	LABEL		First byte of header sector is volume serial
	MOV	C,A
	MVI	B,0		(BC) = Volume number
	MVI	A,3
	CALL	$UDD		Put volume number into message
	CALL	$TYPTX
	DB	NL,'Dumping volume '
MSG.VOL DS	3
	DB	':',' '+200Q

*	Print volume label

	LXI	H,LABEL+LAB.LAB (HL) = addr of label
	MVI	C,60		(C)  = count
PRT.LAB EQU	*
	MOV	A,M
	SCALL	.SCOUT
	INX	H
	DCR	C
	JNZ	PRT.LAB
	MVI	A,NL
	SCALL	.SCOUT

*	Determine the number of tracks and sides

	MVI	A,40
	STA	TRACKS
	STA	STEPS
	MVI	A,1
	STA	SIDES		Set defaults for old disk
	LDA	LABEL+LAB.VER
	CPI	17H
	JC	SETLMTS 	Limits set - ready to go

	LDA	LABEL+LAB.VFL
	ANI	VFL.2SD+VFL.80T Mask out unused bits
	PUSH	PSW
	LXI	H,TRK.TAB
	CALL	$DADA.
	MOV	A,M		(A)  = number of logical tracks
	STA	TRACKS
	POP	PSW
	LXI	H,STP.TAB
	CALL	$DADA.
	MOV	A,M		(A)  = number of physical tracks
	STA	STEPS
	LDA	LABEL+LAB.VFL
	ANI	VFL.2SD
	JZ	SETLMTS 	Single side
	MVI	A,2
	STA	SIDES		Else double
SETLMTS EQU	*
	SPACE	2,10
*	Step #3 - Build the directory extract
*
*	NOTE:	We don't monitor the size of the constructed table.
*		The maximum possible value can't exceed 6K bytes
*		so there's no real point.

	LXI	H,0
	SHLD	DIRUSED 	Init 'used' count
	LXI	H,DIRECT
	SHLD	DIRNEXT 	Next entry to be built
		
	LHLD	LABEL+LAB.DIS	(HL) = first directory sector
DIRLOAD EQU	*
	LXI	D,DIR.BUF	(DE) = buffer address
	LXI	B,512
	MVI	A,DC.REA
	CALL	D.SYDD		Read a block
	JC	ER.REA		
	
*	Look at up to 22 entries in this block

	LHLD	DIRNEXT
	XCHG			(DE) = entry address
	LXI	H,DIR.BUF
	MVI	B,22		Count
ENTLOAD EQU	*
	MOV	A,M		(A) = first char of name
	CPI	376Q
	JZ	DIRDONE 	No more
	CPI	377Q
	JZ	ENT.DEL 	This entry deleted
	
	CALL	LOADENT 	Load this entry into the extract
	JMP	ENTNEXT

*	Entry was deleted or unused - advance (HL)

ENT.DEL EQU	*
	MVI	A,DIR.LEN
	CALL	$DADA.
ENTNEXT EQU	*
	DCR	B		Count down
	JNZ	ENTLOAD 	Keep working on it
	
*	Done with this block. Do the next one.
	
	LHLD	DIR.BUF+DIR.LNK Get link to next block
	JMP	DIRLOAD 	Do the next block

*	No more directory entries to follow

DIRDONE EQU	*
	SPACE	2,10
*	Step #4 - Build the sector dump control table
*
*	A sector can be skipped (part of a group marked in the RGT),
*	or dumped.
*
*	Sectors to be dumped can be part of a binary or text file, which
*	will determine how it's to be filled if there's an I/O error while
*	reading the sector.
*
*	This control table will contain this information on each sector,
*	along with a pointer back to the name of the file, if any, that it's
*	a part of. This information will be dumped in the sector header
*	to aid in dump troubleshooting.
*
*	Mark first 10/12/16 sectors as Binary with no associated file.

	LDA	LABEL+LAB.SPG
	LXI	H,RSRVED
	CALL	$DADA.		(HL) = addr of reserved sector count
	MOV	C,M		(C)  = reserved sector count
	LXI	H,SEC.TAB

*	Clear reserved sectors at front of disk (track 0 and maybe more)

SCT.0	EQU	*
	MVI	M,SEC.BIN	Binary
	INX	H
	MVI	M,0		Clear pointer
	INX	H
	MVI	M,0
	INX	H
	MVI	M,0		Clear relative sector
	INX	H
	MVI	M,0
	INX	H
	DCR	C
	JNZ	SCT.0

*	Mark skipped sectors

	LXI	H,0
	SHLD	FLINK		Directory extract link = 0
	SHLD	FRELSEC 	Relative sector = 0
	LXI	H,RGTOFST
	LDA	LABEL+LAB.SPG
	MOV	C,A		(C)  = SPG
	CALL	$DADA.		(HL) = addr of starting offset
	MOV	L,M
	MVI	H,RGT/256	(HL) = addr of first RGT marker
SCT.1	EQU	*
	MOV	A,M		(A)  = RGT marker
	MVI	B,SEC.FRE	(assume sectors in free list)
	CPI	377Q		Skip?
	JNZ	SCT.2		No
	MVI	B,SEC.SKP	Change code to "skip"
SCT.2	EQU	*
	CALL	SCTTYPE 	Set SCT.TYP field on "SPG" sectors
	INR	L
	MVI	A,310Q
	CMP	L		Done?
	JNZ	SCT.1		No, keep at it

*	Skipped sectors are set, and all others are flagged
*	as being in the free list.
*
*	Now we need to troll through the directory extract and
*	set the type based on the type of the file. We'll also
*	insert the directory extract entry address into the control
*	table.

	LHLD	DIRUSED
	MOV	B,H
	MOV	C,L		(BC) = number of entries
	LXI	H,DIRECT	(HL) = address of first entry
SCT.3	EQU	*
	CALL	SCTFILE 	Set file info in sector table
	DCX	B
	MOV	A,B
	ORA	C
	JNZ	SCT.3

*	We're finally done with the preliminaries.
*	Time to dump the disk.
	SPACE	2,10
*	Step #5 - Write header
*
*	We're going to write the 16-byte file header,
*	then we're going to dump a set of comments including
*	the disk volume and label string and the contents
*	of our directory extract.
	SPACE	2,10
	CALL	$TYPTX
	DB	NL
	DB	'Ready your capture computer to receive '
	DB	'the dumped disk.',NL
	DB	'Press any key to start the disk dump.',NL,ENL
	
	SCALL	.CLRCO
WAITUSR EQU	*
	SCALL	.SCIN
	JC	WAITUSR

*	NOTE:	Do NOT confirm this keystroke! Anything written from
*		this point forward will be part of the captured file,
*		if output is to the console.

*	Write file header

	MVI	A,377Q
	CALL	PUTOCTL
	MVI	A,300Q
	CALL	PUTOCTL 	Header flag
	MVI	A,PROT
	CALL	PUTOCTL 	Protection
	LDA	LABEL
	CALL	PUTOCTL 	Volume serial
	LDA	SIDES
	CALL	PUTOCTL 	Sides
	LDA	STEPS
	CALL	PUTOCTL 	Physical tracks
	MVI	A,10
	CALL	PUTOCTL 	Sectors
	
	MVI	C,9
NULL.LP EQU	*
	XRA	A
	CALL	PUTOCTL 	Nine nulls
	DCR	C
	JNZ	NULL.LP
	MVI	A,NL
	CALL	PUTBYTE

*	Write human-readable disk header

	LDA	LABEL
	MOV	C,A
	MVI	B,0		(BC) = volume serial
	LXI	H,DDESCV
	MVI	A,3
	CALL	$UDD		Put volume into description

	LXI	H,DDESC
	MVI	C,DDESCLN
	CALL	PUTLINE 	Description tag
	LXI	H,LABEL+LAB.LAB
	MVI	C,60
	CALL	PUTLINE 	Add label

*	Add ";Disk is (not) bootable" line

	LXI	H,BOO1
	MVI	C,BOO1.L
	CALL	PUTLINE 	";Disk is "
	LDA	LABEL+LAB.VLT
	CPI	VLT.BOO 	Bootable?
	JZ	IS.BOO		Yes
	LXI	H,BOO2
	MVI	C,BOO2.L
	CALL	PUTLINE 	"not "
IS.BOO	EQU	*
	LXI	H,BOO3
	MVI	C,BOO3.L
	CALL	PUTLINE 	"bootable."

*	Write the directory extract into the disk image

	LXI	H,DIRHEAD
	MVI	C,DIRHEAL
	CALL	PUTLINE

	LHLD	DIRUSED
	XCHG			(DE) = count
	LXI	H,DIRECT
WRITDIR EQU	*
	CALL	DIR.PRT 	Print current entry
	DCX	D
	MOV	A,D
	ORA	E
	JNZ	WRITDIR
	
	CALL	PAMPREP 	Configure front panel
	CALL	PAMTOTL 	Put in total tracks

*	Write sectors

	XRA	A
	STA	TRACK		Init track
	LXI	H,SEC.TAB
	SHLD	SEC.CUR 	Init sector table pointer
GET.TRK EQU	*
	CALL	PAMTRAK 	Display current track
	XRA	A
	STA	SECTOR		Init sector
GET.SEC EQU	*
	CALL	PAMSECT 	Display sector

*	If this entry is to be skipped, we'll be loading it with 000's
*	and will bypass the read.

	LHLD	SEC.CUR
	MOV	A,M
	CPI	SEC.SKP
	JZ	SKIPSEC 	It is skipped

	LDA	TRACK
	MOV	E,A
	MVI	D,0		(DE) = track
	CALL	$MU10
	LDA	SECTOR
	CALL	$DADA.		(HL) = sector to read
	LXI	D,INBUF 	(DE) = buffer
	LXI	B,256
	LDA	INPUT
	SUI	'0'
	STA	AIO.UNI 	Set unit
	LDA	TRACK
	ORA	A		Track zero?
	MVI	A,DC.REA	(assume not)
	JNZ	DOREAD
	MVI	A,DC.RER	Read regardless on track zero
DOREAD	CALL	D.SYDD
	CC	ER.LOG		Log and continue
	JMP	WRITHED
	SPACE	2,10
*	Skipping this sector - fill INBUF with zeros

SKIPSEC EQU	*
	LXI	H,INBUF
	XRA	A
	MOV	C,A
SKIPSE1 EQU	*
	MOV	M,A
	INX	H
	DCR	C		Will do this 256 times
	JNZ	SKIPSE1

	ERRNZ	*-WRITHED
	
*	Write human-readable sector header

WRITHED EQU	*
	LXI	H,TT
	LDA	TRACK
	MOV	C,A
	MVI	B,0
	MVI	A,3
	CALL	$UDD		Put track # in line

	LDA	TRACK
	RAR			'C' set if odd track number
	MVI	C,'0'		Assume even...
	JNC	EVENSID 	Good assumption
	LDA	SIDES
	CPI	1
	JZ	EVENSID 	One-sided drive; go with zero
	MVI	C,'1'		Else, we're on side 1
EVENSID EQU	*
	MOV	A,C
	STA	SD		Set side

	LXI	H,SS
	LDA	SECTOR
	MOV	C,A
	MVI	B,0
	MVI	A,2
	CALL	$UDD		Put sector # in line
	
	LHLD	SEC.CUR
	MOV	A,M
	STA	CT		Add content type
	INX	H
	PUSH	H
	
	LXI	H,SDESC
	MVI	C,SDESCLN
	CALL	PUTLINE 	Add to output
	
	POP	H
	MOV	E,M
	INX	H
	MOV	D,M		(DE) = addr of associated file
	INX	H
	MOV	C,M
	INX	H
	MOV	B,M		(BC) = relative sector
	INX	H
	SHLD	SEC.CUR 	Set for next sector

*	If there's a file associated with this sector,
*	add a file-name line to the comment.

	MOV	A,D
	ORA	E
	JZ	NOFILE		No file - skip it
	
	LXI	H,FILE.RS
	MVI	A,5
	PUSH	D
	CALL	$UDD		Put relative sector into msg
	POP	D
	LXI	H,FILE.RS
	XRA	A
	CALL	BLANK4		Clean up for print

	LXI	H,FILEMSG
	MVI	C,FILEMSL
	CALL	PUTLINE
	XCHG			(HL) = addr of file name
	MVI	C,8
	CALL	PUTLINE
	MVI	A,'.'
	CALL	PUTBYTE
	MVI	C,3
	CALL	PUTLINE
	MVI	A,NL
	CALL	PUTBYTE
NOFILE	EQU	*
	MVI	A,NL
	CALL	PUTBYTE
	
	MVI	B,16		16 lines of output per sector
*				(16*16 = 256 bytes)
	MVI	D,0		Initialize the check byte accumulator
	LXI	H,INBUF
DUMPLIN EQU	*
	MVI	C,16		16 bytes per line
DUMPBYT EQU	*
	MOV	A,M
	PUSH	PSW
	XRA	D
	RLC
	MOV	D,A		Update check byte
	POP	PSW
	CALL	PUTOCTL 	Write a byte
	INX	H
	DCR	C
	JNZ	DUMPBYT
	MVI	A,NL
	CALL	PUTBYTE 	Terminate a line of 16
	DCR	B
	JNZ	DUMPLIN 	Keep this up for 16 lines

*	Done with this sector. Dump the "check" comment then
*	increment and see if more.

	LXI	H,CHKMS
	MVI	C,CHKMSLN
	CALL	PUTLINE
	MOV	A,D		Get check byte
	CALL	PUTOCTL
	MVI	A,NL
	CALL	PUTBYTE

	LDA	SECTOR
	INR	A
	STA	SECTOR
	CPI	10		More sectors to go?
	JC	GET.SEC 	Yes - keep going

*	Done with this track.

	LDA	TRACK
	INR	A		Step up a track
	STA	TRACK
	LXI	H,TRACKS
	CMP	M
	JC	GET.TRK 	More to go

*	Ordinarily, we might not flush the buffer at this point as we might
*	have more disks to come. But if the user is capturing this on the
*	console we need to flush before writing any further messages to
*	make sure we have a clean dump.

	CALL	FLUSH
	
*	Ring the bell a few times to signal completion

	MVI	C,5
DING	EQU	*
	MVI	A,BEL
	SCALL	.SCOUT
	CALL	PAUSE
	DCR	C
	JNZ	DING

	LHLD	ERR.CNT
	MOV	B,H
	MOV	C,L
	LXI	H,ERRORS
	MVI	A,5
	CALL	$UDD		Put errors in message
	LXI	H,ERRORS
	XRA	A		(fill with nulls)
	CALL	BLANK4
	
	CALL	$TYPTX
	DB	NL,'Done dumping disk - '
ERRORS	DS	5
	DB	' errors encountered',NL
	DB	'Please remove disk from drive',ENL
	
	LDA	INPUT
	SUI	'0'
	STA	AIO.UNI 	Set unit
	SCALL	.CLRCO
WAIT.3	EQU	*		Wait for disk to be removed
	SCALL	.SCIN
	JC	WAIT.4
	CPI	4		Ctl-D?
	JZ	REMOUNT
WAIT.4	EQU	*
	MVI	A,DC.RDY
	CALL	D.SYDD
	JNC	WAIT.3		Still in - wait for removal
	
	JMP	ASK
	SPACE	2,10
*	CTLCXIT User typed ctl-c while we were dumping a disk.

CTLCXIT EQU	*
	CALL	FLUSH

	CALL	$TYPTX
	DB	NL,BEL,'Dump canceled - do not use the incomplete output!',ENL

	ERRNZ	*-REMOUNT

*	User quit with a disk in the drive - mount it.

REMOUNT EQU	*
	LXI	H,SY.SPEC
	SCALL	.MOUNT
	ERRNZ	*-CLOSE

CLOSE	EQU	*		Close output file
	XRA	A
	SCALL	.CLOSE
	JMP	EXIT
	STL	'Front Panel management routines'
	EJECT
*	PAMPREP - Prep front panel for private display
*
*	Entry:	None
*
*	Exit:	None
*
*	Uses:	A,F

PAMPREP EQU	*
	LDA	.MFLAG
	ORI	UO.DDU		Disable update
	STA	.MFLAG
	MVI	A,377Q
	STA	DLEDS		Blank outer data leds
	STA	DLEDS+2
	RET
	SPACE	2,10
*	PAMRSET - Reset front panel
*
*	Entry:	None
*
*	Exit:	None
*
*	Uses:	A,F

PAMRSET EQU	*
	LDA	.MFLAG
	ANI	377Q-UO.DDU
	STA	.MFLAG
	RET
	SPACE	2,10
*	PAMTOTL - Display total tracks in left-most ALEDS
*
*	Entry:	None
*
*	Exit:	None
*
*	Uses:	All

PAMTOTL EQU	*
	LXI	D,ALEDS
	LDA	TRACKS
	JMP	PAMOUT3
	SPACE	2,10
*	PAMTRAK - Display current track in right-most ALEDS
*
*	Entry:	None
*
*	Exit:	None
*
*	Uses:	All

PAMTRAK EQU	*
	LXI	D,ALEDS+3
	LDA	TRACK
*
	ERRNZ	*-PAMOUT3
	SPACE	2,10
*	PAMOUT3 - Write 3 decimal digits of output
*
*	Entry:	(DE) = address of LED output
*		(A)  = value
*
*	Exit:	None
*
*	Uses:	All

PAMOUT3 EQU	*
	PUSH	D
	LXI	H,FPWORK
	MOV	C,A
	MVI	B,0
	MVI	A,3
	CALL	$UDD		Convert value to decimal
	POP	D
	PUSH	D
	LXI	H,FPWORK
	MVI	C,3
PAMOUTA EQU	*
	PUSH	H
	MOV	A,M
	SUI	'0'		Convert digit to binary
	LXI	H,FP.TAB
	CALL	$DADA.		(HL) = Addr of bit pattern
	MOV	A,M
	STAX	D
	POP	H
	INX	H
	INX	D
	DCR	C
	JNZ	PAMOUTA

*	Blank up to two leading zeros

	POP	D
	LXI	H,FP.TAB	(HL) = address of zero pattern
	MVI	C,2
PAMOUTB EQU	*
	LDAX	D
	CMP	M		Zero?
	RNZ			No - done
	MVI	A,377Q
	STAX	D		Blank the LED
	INX	D
	DCR	C
	JNZ	PAMOUTB
	RET
	SPACE	2,10
*	PAMSECT - Put sector number in middle of DLEDS
*
*	Entry:	None
*
*	Exit:	None
*
*	Uses:	A,F,H,L

PAMSECT EQU	*
	LDA	SECTOR
	LXI	H,FP.TAB
	CALL	$DADA.		(HL) = address of pattern
	MOV	A,M
	STA	DLEDS+1
	RET
	STL	'Directory routines'
	EJECT
*	LOADENT - Load an entry into the directory extract
*
*	Entry:	(HL) = Addr of entry in REAL directory
*
*	Exit:	(HL) = Advanced past that entry
*
*	Uses:	A,F,H,L
*
LOADENT EQU	*
	PUSH	B
	PUSH	D
	XCHG
	LHLD	DIRNEXT
	LXI	B,11
	CALL	$MOVE		Load name and extension
	XCHG
	LXI	B,DIR.FGN-DIR.EXT-3
	DAD	B		(HL) = addr of DIR.FGN
	MOV	A,M		(A)  = FGN
	STAX	D		Store in DE.FGN
	INX	D		(DE) = addr of DE.SIZ
	INX	H
	INX	H		(HL) = addr of DIR.LSI
	CALL	FILESIZ 	(BC) = file size in sectors
	XCHG
	MOV	M,C
	INX	H
	MOV	M,B		Save size
	INX	H
	SHLD	DIRNEXT
	LHLD	DIRUSED
	INX	H		Count this entry
	SHLD	DIRUSED
	XCHG
	LXI	B,DIR.LEN-DIR.LSI
	DAD	B		(HL) = advanced to next entry
	POP	D
	POP	B
	RET
	SPACE	2,10
*	DIR.PRT - "Print" one directory extract into the output file
*
*	Entry	- (HL) = addr of extract entry
*
*	Exit	- (HL) = advanced to next entry
*
*	Uses	- A,F,H,L
*
DIR.PRT EQU	*
	PUSH	B
	PUSH	D
	MVI	A,';'
	CALL	PUTBYTE
	MVI	A,' '
	CALL	PUTBYTE
	MVI	C,8
	CALL	DIRPPRT 	Write name
	MVI	A,'.'
	CALL	PUTBYTE
	MVI	C,3
	CALL	DIRPPRT 	Write extension

*	Get file type

	PUSH	H
	MOV	L,M		(L)  = FGN
	LDA	LABEL+LAB.SPG
	MOV	C,A		(C)  = SPG
	CALL	SCTOFST 	(HL) = addr of SEC.TAB entry
	MOV	A,M
	STA	FSM.TYP
	POP	H
	INX	H
	MOV	C,M
	INX	H
	MOV	B,M		(BC) = file size
	INX	H		(HL) = addr of next entry
	PUSH	H
	LXI	H,FSM.SEC
	MVI	A,5
	CALL	$UDD		Convert size to decimal
	LXI	H,FSM.SEC
	MVI	A,' '		(fill with spaces)
	CALL	BLANK4		Blank leading zeros
	LXI	H,FSM.MSG
	MVI	C,FSM.LEN
	CALL	PUTLINE 	Write rest of line
	POP	H
	POP	D
	POP	B
	RET

*	Write characters to the output buffer,
*	replacing nulls with spaces.

DIRPPRT EQU	*
	MOV	A,M
	ORA	A
	JNZ	DIRP.1
	MVI	A,' '
DIRP.1	CALL	PUTBYTE
	INX	H
	DCR	C
	JNZ	DIRPPRT
	RET
	
*	End of directory message
	
FSM.MSG EQU	*
FSM.SEC DS	5		File size in sectors
	DB	'   '
FSM.TYP DS	1		File type
	DB	NL
FSM.LEN EQU	*-FSM.MSG
	SPACE	2,10
*	FILESIZ - Calculate a file size in sectors
*
*	Entry:	(A)  = FGN
*		(HL) = addr of DIR.LSI
*
*	Exit:	(BC) = file size in sectors
*
*	Uses:	A,F,B,C
*
FILESIZ EQU	*
	PUSH	H
	LXI	H,GRT
	MOV	L,A		(HL) = addr of first link
	LXI	B,0
FS.1	EQU	*
	MOV	A,M
	ORA	A		Another link?
	JZ	FS.2		No - done
	MOV	L,A		(HL) = addr of next link
	LDA	LABEL+LAB.SPG	Get sectors/group
	CALL	FS.3		Update sector count
	JMP	FS.1

FS.2	EQU	*
	POP	H		(HL) = addr of DIR.LSI
	MOV	A,M

*	Update the sector count

FS.3	EQU	*
	ADD	C		Update sector count
	MOV	C,A
	RNC			No overflow
	MVI	A,0
	ADC	B
	MOV	B,A
	RET
	SPACE	2,10
*	BLANK4	- Black up to four leading zeros
*
*	Entry:	(HL) = addr of numeric string
*		(A)  = fill character
*
*	Exit:	none
*
*	Uses:	A,F,H,L
*
BLANK4	EQU	*
	PUSH	B
	MOV	B,A		(B)  = fill char
	MVI	C,4		(C)  = count
BLANK4A EQU	*
	MOV	A,M
	CPI	'0'
	JNZ	BLANK4X
	MOV	M,B		Replace '0' with fill char
	INX	H
	DCR	C
	JNZ	BLANK4A
BLANK4X EQU	*
	POP	B
	RET
	STL	'Sector control table routines'
	EJECT
*	SCTTYPE - Set the sector control table SCT.TYP field
*
*	Entry:	(B)  = type to set
*		(C)  = SPG
*		(L)  = group number
*
*	Exit:	none
*
*	Uses:	A,F
*
SCTTYPE EQU	*
	PUSH	H
	PUSH	D
	PUSH	B
	CALL	SCTOFST 	(HL) = sector table entry addr
SCTTYP1 EQU	*
	MOV	M,B		Update type
	INX	H
	XCHG
	LHLD	FLINK
	XCHG
	MOV	M,E		Store file name pointer
	INX	H
	MOV	M,D
	INX	H		(HL) = addr of next entry
	XCHG
	LHLD	FRELSEC
	INX	H
	SHLD	FRELSEC 	Update for next sector
	DCX	H
	XCHG
	MOV	M,E		Store relative sector
	INX	H
	MOV	M,D
	INX	H
	DCR	C
	JNZ	SCTTYP1 	Continue for SPG sectors
	POP	B
	POP	D
	POP	H
	RET
	SPACE	2,10
*	SCTOFST - Calculate the address of the SEC.TAB entry
*		  given the group and SPG.
*
*	Entry:	(C)  = SPG
*		(L)  = group number
*
*	Exit:	(HL) = addr of SEC.TAB entry
*
*	Uses:	A,F,H,L
*
SCTOFST EQU	*
	PUSH	B
	PUSH	D
	MVI	D,0
	MOV	E,L
	MOV	A,C
	CALL	$MU86		(HL) = first sector table index
	XCHG
	MVI	A,SEC.LEN
	CALL	$MU86		(HL) = first sector table offset
	LXI	D,SEC.TAB
	DAD	D		(HL) = first sector address
	POP	D
	POP	B
	RET
	SPACE	2,10
*	SCTFILE - Update the sector control table from a directory entry
*
*	Entry:	(HL) = addr of directory extract entry
*
*	Exit:	(HL) = advanced to next entry
*
*	Uses:	A,F,H,L
*
SCTFILE EQU	*
	PUSH	B
	PUSH	D
	PUSH	H
	
	SHLD	FLINK		Save link for type insert
	LXI	H,0
	SHLD	FRELSEC 	Init relative sector
	POP	H		Refresh (HL)
	PUSH	H

*	Check type against table

	MVI	A,8
	CALL	$DADA.
	XCHG			(DE) = addr of DE.EXT
	LXI	H,TYPETAB
	MVI	B,TYPECNT
	MVI	C,3		(C)  = compare length
SCTFIL1 EQU	*
	CALL	COMP		
	JZ	SCTFIL2 	Match on extension
	MVI	A,TYPETLN
	CALL	$DADA.		Step to next entry
	DCR	B
	JNZ	SCTFIL1

*	No go on extension. Try 1st byte of file

	LXI	H,DE.FGN-DE.EXT
	DAD	D		(HL) = addr of DE.FGN
	PUSH	H
	MOV	E,M
	MVI	D,0		(DE) = first group
	LDA	LABEL+LAB.SPG	(A)  = SPG
	CALL	$MU86		(HL) = first sector index
	LXI	D,INBUF
	LXI	B,256
	MVI	A,DC.REA
	CALL	D.SYDD		Read first sector
	POP	H		(HL) = addr of DE.FGN
	JC	SCTFIL3 	Bad read - use binary
	LDA	INBUF
	CPI	377Q
	JZ	SCTFIL3 	It's binary

*	It's a text file

	MVI	B,SEC.TXT
	JMP	SCTFIL4

*	We had a hit in the type table.
*	1) Type to (B).
*	2) Advance (HL) to DE.FGN.

SCTFIL2 EQU	*
	MVI	A,3
	CALL	$DADA.
	MOV	B,M		(B)  = type from table
	LXI	H,DE.FGN-DE.EXT
	DAD	D		(HL) = addr of DE.FGN
	JMP	SCTFIL4

*	I/O error or first byte = 377Q. Type=Binary.

SCTFIL3 EQU	*
	MVI	B,SEC.BIN

*	Set the type

SCTFIL4 EQU	*
	MOV	A,M
	ORA	A		Done?
	JZ	SCTFILX 	Yes
	MOV	L,A		(L)  = DE.FGN
	PUSH	H
	LDA	LABEL+LAB.SPG
	MOV	C,A		(C)  = SPG
	CALL	SCTTYPE 	Set the type
	POP	H
	MVI	H,GRT/256	Move to GRT from the directory extract
	JMP	SCTFIL4 	... and go again

SCTFILX EQU	*
	POP	H
	LXI	D,DE.LEN
	DAD	D		(HL) = addr of next dir extract entry
	POP	D
	POP	B
	RET

*	Extension / Type table

TYPETAB EQU	*
	DB	'TXT',SEC.TXT
TYPETLN EQU	*-TYPETAB		Size of one entry
	DB	'ASM',SEC.TXT
	DB	'ACM',SEC.TXT
	DB	'BAS',SEC.TXT
	DB	'ABS',SEC.BIN
	DB	'DVD',SEC.BIN
	DB	'SYS',SEC.BIN
	DB	'BIN',SEC.BIN
	DB	'DAT',SEC.TXT
TYPECNT EQU	*-TYPETAB/TYPETLN	# of entries
	STL	'Misc. Utilities'
	EJECT
*	COMP	- Compare two strings
*
*	Entry:	(DE) = string addr
*		(HL) = string addr
*		(C)  = length
*
*	Exit:	'Z' set if equal
*
*	Uses:	A,F
*
COMP	EQU	*
	PUSH	B
	PUSH	D
	PUSH	H
COMP.1	EQU	*
	LDAX	D
	CMP	M
	JNZ	COMP.X
	INX	H
	INX	D
	DCR	C
	JNZ	COMP.1
COMP.X	EQU	*
	POP	H
	POP	D
	POP	B
	RET
	SPACE	2,10
*	PAUSE	- Wait about 1/2 second
*
*	Entry:	none
*
*	Exit:	none
*
*	Uses:	A,F,H,L
*
PAUSE	EQU	*
	LXI	H,TICCNT
	MOV	A,M
PAUSE1	EQU	*
	CMP	M		Same?
	JZ	PAUSE1		Yes - wait for change
PAUSE2	EQU	*
	CMP	M		Same?
	JNZ	PAUSE2		No
	RET
	STL	'Output routines'
	EJECT
*	PUTLINE - Write a line of text
*
*	Entry:	(HL) = Addr of text
*		(C)  = Count
*
*	Exit:	None
*
*	Uses:	H,L,C,A,F
*
PUTLINE EQU	*
	MOV	A,M
	CALL	PUTBYTE
	INX	H
	DCR	C
	JNZ	PUTLINE
	RET
	SPACE	2,10
*	PUTOCTL - Write a byte as three octal digits and a space.
*
*	Entry:	(A)  = byte
*
*	Exit:	none
*
*	Uses:	A,F
*
PUTOCTL EQU	*
	PUSH	B
	MVI	C,3
	ORA	A		Pre-clear carry flag
PUTOC.1 EQU	*
	RAL
	RAL
	RAL			Target bits are 2-0
	PUSH	PSW
	ANI	00000111B	Isolate them
	ADI	'0'		Shift to ascii
	CALL	PUTBYTE 	Write it
	POP	PSW
	DCR	C
	JNZ	PUTOC.1
	MVI	A,' '
	CALL	PUTBYTE 	Send a space
	POP	B
	RET
	SPACE	2,10
*	FLUSH	- Flush output buffer
*
*	Fill the last sector with nulls and force it to be
*	written to the output serial device.
*
*	Entry:	none
*
*	Exit:	none
*
*	Uses:	all
*
FLUSH	EQU	*
	LXI	H,OUTNDX
FLUSH.1 EQU	*
	MOV	A,M
	ORA	A
	RZ			Last buffer written
	XRA	A
	CALL	PUTBYTE 	Else write a null
	JMP	FLUSH.1 	...and try again
	SPACE	2,10
*	PUTBYTE - Put a byte in the output buffer.
*
*	Entry:	(A)  = byte to output
*
*	Exit:	none
*
*	Uses:	A,F
*
PUTBYTE EQU	*
	PUSH	B
	PUSH	D
	PUSH	H
	LHLD	OUTPTR
	MOV	M,A		Put byte in buffer
	INX	H
	SHLD	OUTPTR
	LDA	OUTNDX
	DCR	A		Count it
	STA	OUTNDX
	JNZ	PUT.X
*
*	Buffer is full - write it
*
	LXI	H,OUTBUF
	SHLD	OUTPTR		Reset ptr
	XCHG			(DE) = buffer
	LXI	B,256
	XRA	A
	SCALL	.WRITE
	JC	ER.WRIT 	Not good
PUT.X	EQU	*
	POP	H
	POP	D
	POP	B
	RET
	STL	'Errors and Exit'
	EJECT
*	ER.LOG	- Log and count this error.
*
*	Entry:	none
*
*	Exit:	none
*
*	Uses:	all
*
ER.LOG	EQU	*
	LHLD	ERR.CNT
	INX	H
	SHLD	ERR.CNT 	Update statistics
	LHLD	SEC.CUR
	MOV	A,M		(A)  = sector type
	LXI	H,INBUF
	MVI	C,128		Preload loop count
	
	CPI	SEC.BIN
	JZ	ER.BIN
	CPI	SEC.TXT
	JZ	ER.TXT
	
*	Other than binary or text

	LXI	D,0		Fill with 000 000 000 ...
	JMP	ER.FIL
	
*	Binary

ER.BIN	EQU	*
	MVI	D,377Q
	MVI	E,000Q		Fill with SCALL .EXIT's
	JMP	ER.FIL

*	Text

ER.TXT	EQU	*
	MVI	D,'?'
	MOV	E,D		Fill with "????"

ER.FIL	EQU	*
	MOV	M,D
	INX	H
	MOV	M,E
	INX	H
	DCR	C
	JNZ	ER.FIL
	
	LXI	H,IOERM
	MVI	C,IOERML
	CALL	PUTLINE 	Log message
	
	RET
	SPACE	2,10
*	Fatal error messages and EXIT code.

ER.MOU	EQU	*		Problem with driver mount
	CALL	$TYPTX
	DB	BEL,'Disk would not mount - aborting.',ENL
	JMP	EXIT

ER.REA	EQU	*		Problem reading
	PUSH	PSW
	CALL	$TYPTX
	DB	NL,BEL,'Error reading disk - aborting.',ENL
	POP	PSW
	MVI	H,NL
	SCALL	.ERROR
	JMP	EXIT

ER.WRIT EQU	*		Problem writing
	PUSH	PSW
	CALL	$TYPTX
	DB	NL,BEL,'Error writing output - aborting.',ENL
	POP	PSW
	MVI	H,NL
	SCALL	.ERROR		Give 'em the message
	JMP	EXIT

BADVERS EQU	*		HDOS 2.0 required
	CALL	$TYPTX
	DB	NL,BEL,'HDOS 2.0 is required - aborting.',ENL

	ERRNZ	*-EXIT

EXIT	EQU	*
	XRA	A
	SCALL	.EXIT
	STL	'Work Space'
	EJECT
*	Work Space

TRACK	DS	1		Track number (0-TRACKS)
TRACKS	DS	1		40, 80, or 160
STEPS	DS	1		40 or 80 (goes in header)
SIDES	DS	1		Number of sides (1 or 2)

SECTOR	DS	1		Sector number (0-9)
OUTPUT	DS	10		Output serial device
OUTDFLT DB	'TT0',0,0,0	Default block for output open

SY.SPEC DB	'SY'
INPUT	DS	1		Input disk unit number
	DB	':',0

TRK.TAB DB	40,80,80,160	# of logical tracks for different VFL bits
STP.TAB DB	40,40,80,80	# of physical tracks for different VFL bits

*	Front Panel digit table

FP.TAB	DB	10000001B,11110011B	0, 1
	DB	11001000B,11100000B	2, 3
	DB	10110010B,10100100B	4, 5
	DB	10000100B,11110001B	6, 7
	DB	10000000B,10110000B	8, 9

FPWORK	DS	3		Digit decode work space

DDESC	DB	NL,';Volume: '
DDESCV	DS	3
	DB	NL,';Description: '
DDESCLN EQU	*-DDESC

BOO1	DB	NL,';Disk is '
BOO1.L	EQU	*-BOO1

BOO2	DB	'not '
BOO2.L	EQU	*-BOO2

BOO3	DB	'bootable',NL
BOO3.L	EQU	*-BOO3

DIRHEAD DB	NL,'; DIRECTORY',NL
	DB	'; File Name  '
	DB	'  Size Type',NL,NL
DIRHEAL EQU	*-DIRHEAD				

SDESC	DB	NL,';Data: Track '
TT	DS	3
	DB	', Side '
SD	DS	1
	DB	', Sector '
SS	DS	2
	DB	', Content type '
CT	DS	1
	DB	NL
SDESCLN EQU	*-SDESC

CHKMS	DB	NL,';check: '	Note: case sensitive; must be lower case
CHKMSLN EQU	*-CHKMS

FLINK	DS	2		Link to file name for sector insert
FRELSEC DS	2		Link to relative sector for sector insert

FILEMSG DB	';Relative sector '
FILE.RS DS	5
	DB	' of file '
FILEMSL EQU	*-FILEMSG

IOERM	DB	NL,';I/O Error - Sector filled'
IOERML	EQU	*-IOERM

*	Depending on the sectors/group, we have to start inspecting
*	the RGT at different offsets from the front. This table is
*	in SPG order and gives the starting RGT offsets.

RGTOFST DB	0,0,5,0,3,0,0,0,2

*	Depending on the sectors/group, we need to flag the first n
*	sectors as binary-no file in the sector control table:
*		spg=2 - 5 groups / 10 sectors reserved
*		spg=4 - 3 groups / 12 sectors reserved
*		spg=8 - 2 groups / 16 sectors reserved
*	This table gives us the reserved count. It's in SPG order.

RSRVED	DB	0,0,10,0,12,0,0,0,16

LASTCON EQU	*		All DS space below here

LABEL	DS	256		Disk label

ERR.CNT DS	2		I/O error count

OUTBUF	DS	256		Serial output buffer
OUTPTR	DS	2		Serial output next-byte pointer
OUTNDX	DS	1		Index to output buffer

DIR.BUF DS	512		Directory block buffer
INBUF	EQU	DIR.BUF 	Sector input buffer

*	We need to have the GRT and RGT sectors on addresses evenly
*	divisible by 256. The following stmts will align us to an
*	appropriate boundary.

.	SET	#*		Get low byte of current address
	IF	.		If already zero, cool
	ELSE
	DS	256-.		Align to proper boundary
	ENDIF

	ERRNZ	#*		Verification
	
GRT	DS	256		GRT from disk
RGT	DS	256		RGT from disk

SEC.TAB DS	SEC.LEN*SEC.CNT Sector type table
SEC.CUR DS	2		Currently-used entry

MAX.MEM DS	2		Maximum memory available

DIRUSED DS	2		Directory entries used
DIRNEXT DS	2		Next space to be used

DIRECT	EQU	*		Directory beginning

	ORG	LASTCON 	Don't save buffer space with .abs file

	END	BEGIN
